<?php 
class Image
	{
		const
			IMAGEINITED=0x1,
			REGISTERED=0x2,
			CHECKED = 0x4,
			TEMPORARY=0x8,
			MARKED=0x10;

		static protected
			$p_inited,
			$p_nogd=false,
			$p_support=array('gif'=>1, 'gif_create'=>1, 'jpeg'=>2, 'png'=>3, 'bmp'=>4, 'xbm'=>5);
			
		protected
			$BITS, // 0x1 - is_image, 0x2 - is resampled
			$p_ops=array(),
			$p_id,
			$p_res,
			$p_file,
			$p_dir,
			$p_size,

			$p_thumb,
			$p_thumb_width,
			$p_thumb_height,
			$p_thumb_res=null,

			$p_date,
			$p_width,
			$p_height,
			$p_type,
			$p_flags;

		private function INIT()
			{
				if (!self::$p_inited)
					{
						if (extension_loaded('gd'))
							{
								$a=gd_info();
								if (!$a['GIF Read Support']) unset(self::$p_support['gif']);
								if (!$a['GIF Create Support']) unset(self::$p_support['gif_create']);
								if (!$a['PNG Support']) unset(self::$p_support['png']);
								if (!$a['WBMP Support']) unset(self::$p_support['bmp']);
								if (!$a['JPG Support']) unset(self::$p_support['jpg']);
								if (!$a['XBM Support']) unset(self::$p_support['xbm']);
							}
						else
							self::$p_nogd=true;
						self::$p_inited=true;
					}
			}

		static function GET_FILE($_file)
			{
				self::INIT();
				$file=FLEX::CANONICAL_NAME($_file, true);
				if (false!==$F=getimagesize($file))
					{
						$path=pathinfo($_file);
						$A=array(
							'file'=>$path['basename'],
							'dir'=>$path['dirname'],
							'type'=>$F['mime'],
							'width'=>$F[0],
							'height'=>$F[1],
							'size'=>filesize($file)
							);
						return new Image($A);
					}
				return null;
			}

		static function COPY_RESIZED($_src, &$_width, &$_height=null, $_old_width=null, $_old_height=null, $_prop=false, $_resampled=false)
			{
				if (!$_src)
					return;

				if ($_old_width===null)
					$_old_width=imagesx($_src);
				if ($_old_height===null)
					$_old_height=imagesy($_src);

				if ($_height===null)
					{
						settype($_width, 'float');
						$k=$_width;
						$_width=$_old_width*$k;
						$_height=$_old_height*$k;
					}
				else
					{
						settype($_width, 'int');
						settype($_height, 'int');
					}

				if ($_prop)
					{
						$d = $_old_width / $_old_height;
						if (($_old_width / $_width) < ($_old_height / $_height))
							$_width=$_height * $d;
						else
							$_height=$_width / $d;
					}

				if (!$_width=round($_width))
					$_width=1;
				if (!$_height=round($_height))
					$_height=1;
				
				if ($new_img=imagecreatetruecolor($_width, $_height))
					{
						if ($_resampled)
							$res=@imagecopyresampled($new_img, $_src, 0, 0, 0, 0, $_width, $_height, $_old_width, $_old_height);
						else
							$res=@imagecopyresized($new_img, $_src, 0, 0, 0, 0, $_width, $_height, $_old_width, $_old_height);
						if ($res)
							{
								return $new_img;
							}
						else
							{
								imagedestroy($new_img);
								return null;
							}
					}
			}

		static function FLUSH($_res, $_type, $_quality=75)
			{
				if (!headers_sent())
					header('Content-type: image/jpeg', true);
				switch($_type)
					{
						case 'image/gif':
							imagecolortransparent($_res);
							return imagegif($_res);
						case 'image/bmp':
							return imagewbmp($_res);
						case 'image/png':
							return imagepng($_res);
						case 'image/jpeg':
						default:
							return imagejpeg($_res, null, $_quality);
					}
			}

		static protected function P_OUTPUT($_file, $_res, $_type, $_quality=75)
			{
				switch($_type)
					{
						case 'image/gif':
							imagecolortransparent($_res);
							return $_file ? imagegif($_res, $_file) : imagegif($_res) ;
						case 'image/jpeg':
							return imagejpeg($_res, $_file, $_quality);
						case 'image/bmp':
							return imagewbmp($_res, $_file);
						case 'image/png':
							return $_file ? imagepng($_res, $_file) : imagepng($_res);
						default:
							return false;
					}
			}

		protected function __construct($_array, $_standalone=false)
			{
				$this->pGather($_array);
			}

		function __destruct()
			{
			}

		function __get($_n)
			{
				switch($_n)
					{

						case 'src':
							return $this->p_dir.'/'.$this->p_file;
						
						case 'file':
							return $this->p_file;

						case 'directory':
							return $this->p_dir;

						case 'size':
							if (!$this->p_size && $this->p_file)
								filesize(FLEX::CANONICAL_NAME($this->p_dir.'/'.$this->p_file, true));
							return $this->p_size;

						case 'width':
							if (!$this->p_width && $this->BITS & self::IMAGEINITED)
									$this->p_width=@imagesx($this->p_res);
								return $this->p_width;
						case 'height':
							if (!$this->p_height && $this->BITS & self::IMAGEINITED)
									$this->p_height=@imagesy($this->p_res);
								return $this->p_height;

						case 'type':
							return $this->p_type;
						
						case 'flags':
							return $this->p_flags;
							

						case 'is_image':
							return (bool) $this->p_type;

						default:
							if (isset($this->p_ops[$_n]))
								return $this->p_ops[$_n];
					}
			}

		function __set($_n, $_v)
			{
				switch($_n)
					{
						case 'type':
								switch($_v)
									{
										case 'image/gif':
										case 'image/jpeg':
										case 'image/bmp':
										case 'image/png':
											if (self::$p_support[substr($_v, 6)])
												$this->p_type=$_v;
											break;
									}
							break;
						case 'flags':
								$this->p_flags=(int)$_v;
								$this->BITS |= self::MARKED;
							break;

						case 'date':
							break;

						default:
							$this->p_ops[$_n]=$_v;
					}
			}

		function getThumb($_width, $_height, $_proportional=false, $_resampled=false, $_path=null)
			{
				if (!($this->thumb && file_exists(FLEX::CANONICAL_NAME($this->thumb, true))))
					{
						if (!($this->BITS & IMAGE::IMAGEINITED && $this->p_res))
							$this->pLoadFile();
						if ($this->p_res)
							{
								if ($this->p_width > $_width || $this->p_height > $_height)
									$thumb=self::COPY_RESIZED($this->p_res, $_width, $_height, $this->p_width, $this->p_height, $_proportional, $_resampled);
								else
									{
										$this->p_ops['thumb_type']=$this->p_type;
										$thumb=$this->p_res;
										$_width=$this->p_width;
										$_height=$this->p_height;
									}
								if ($thumb)
									{
										$this->p_thumb_res=$thumb;
										$this->p_thumb_width=$_width;
										$this->p_thumb_height=$_height;
										if ($_path)
											{
												$file=FLEX::CANONICAL_NAME($_path.'/'.$this->p_file.substr(APPLICATION::GENERATE_ID($this->p_file.QUASIS_SYSTEM_TIME, '', true), 0, 15).'.jpeg');
												if (imagejpeg($thumb, FLEX::CANONICAL_NAME($file, true), ($q=(int)$this->p_ops['thumb_quality']) ? $q : 80))
													$this->p_thumb=$file;
											}
										else
											return $thumb;
										imagedestroy($thumb);
										return true;
									}
							}
					}
				return false;
			}

		function output($_file='')
			{
				if (!($this->BITS & IMAGE::IMAGEINITED && $this->p_res))
					$this->pLoadFile();
				if ($this->p_res);
					self::P_OUTPUT($_file, $this->p_res, $this->p_type, ($q=(int)$this->p_ops['quality']) ? $q : 75);
				return false;
			}

		function outputThumb($_file='')
			{
				if (!$this->p_thumb_res && $this->p_thumb)
					{
						$this->p_thumb_res=imagecreatefromjpeg(FLEX::CANONICAL_NAME($this->p_thumb, true));
					}
				if ($this->p_thumb_res)
					{
						self::P_OUTPUT($_file, $this->p_thumb_res, $_ENV->content_type, ($q=(int)$this->p_ops['thumb_quality']) ? $q : 75);
						imagedestroy($this->p_thumb_res);
					}
			}
		
		protected function pGather(&$_array)
			{
				$this->p_id=$_array['image_id'];
				if ($_array['width'])
					$this->p_width=$_array['width'];
				if ($_array['height'])
					$this->p_height=$_array['height'];

				$this->p_thumb=$_array['thumb'];
				$this->p_thumb_height=$_array['thumb_height'];
				$this->p_thumb_width=$_array['thumb_width'];
				
				$this->p_date=$_array['create_date'];

				if ($_array['type'])
					$this->p_type=$_array['type'];
				$this->p_file=$_array['file'];
				$this->p_dir=FLEX::CANONICAL_NAME($_array['dir']);
				$this->p_size=$_array['size'];

				$this->p_flags=$_array['flags'];

				if ($bits=(int)$_array['bits'])
					$this->BITS |= $bits;
			}

		protected function pLoadFile()
			{

				switch($this->p_type)
					{
						case 'image/gif':
							if (self::$p_support['gif'])
								$this->p_res=imagecreatefromgif(FLEX::CANONICAL_NAME($this->p_dir.'/'.$this->p_file, true));
							break;
						case 'image/jpeg':
							if (self::$p_support['jpeg'])
								$this->p_res=imagecreatefromjpeg(FLEX::CANONICAL_NAME($this->p_dir.'/'.$this->p_file, true));
							break;
						case 'image/png':
							if (self::$p_support['png'])
								$this->p_res=imagecreatefrompng(FLEX::CANONICAL_NAME($this->p_dir.'/'.$this->p_file, true));
							break;
						case 'image/bmp':
							if (self::$p_support['bmp'])
								$this->p_res=imagecreatefromwbmp(FLEX::CANONICAL_NAME($this->p_dir.'/'.$this->p_file, true));
							break;
						default:
							return false;
					}
				$this->BITS |=self::IMAGEINITED;
				if ($this->p_res)
					{
						$this->p_width=imagesx($this->p_res);
						$this->p_height=imagesy($this->p_res);
					}
			}
		
		function free()
			{
				if ($this->p_res)
					{
						$this->p_width=imagesx($this->p_res);
						$this->p_height=imagesy($this->p_res);
						imagedestroy($this->p_res);
					}
			}

		function destroy()
			{
				if (is_file($file=FLEX::CANONICAL_NAME($this->src, true)))
					{
						FLEX::DELETE_FILE($file);
					}
				if ($this->p_thumb && is_file($file=FLEX::CANONICAL_NAME($this->thumb, true)))
					{
						FLEX::DELETE_FILE($file);
					}
				$this->p_file=null;
				$this->p_dir=null;
				$this->p_id=null;
				$this->p_size=null;
				$this->p_thumb=null;
				$this->p_thumb_width=null;
				$this->p_thumb_height=null;
				$this->p_date=null;
			}
	}
?>