
<?php
#################################################################
#
# Ldap Class
# "ldap.php" by Dmitriy, dmitriy.zherlygin@ileader.ru
#
#################################################################

class Ldap {

    /*  Ldap class constructor  */
    function Ldap($config) {

        $this->config  = $config; // set config
        $this->ldapUrl = $this->config['serverHost'].":".$this->config['serverPort'];  // set connection url
        $this->prepare();
    }

    /* rovide connection info */ 
    function info() {
        $infoHash = array (
            "ldap_admin"  => $this->config['userName'],
            "ldap_url"    => $this->ldapUrl,
            "ldap_domain" => $this->config['domain'],
            "ldap_dn"     => $this->config['distName'],
            "ldap_import_fields" => $this->importAttr
            );
        
        return $infoHash;
    }
    
    /* ldap server connect */
    function connect() {

        // check for config
        if (! $this->config) return FALSE;
        
        // connect
        if ($this->conn = ldap_connect($this->ldapUrl)) {
            print "Sdfdfgdfg";
            // set connection options
            ldap_set_option($this->conn, LDAP_OPT_PROTOCOL_VERSION, $this->config['protocolVersion']); 
            ldap_set_option($this->conn, LDAP_OPT_REFERRALS, 0);

            // bind connection
            if (!$this->_bind()) return FALSE;

            return TRUE;
        } else 
            return FALSE;
    }

    /* function check login */
    function checkLogin ($user, $passwd) {

        //check for config
        if (! $this->config) return FALSE;

        $auth_user = $user."@".$this->config['domain'];

        // connect to ldap server
        if($this->conn = @ldap_connect($this->ldapUrl)) {

            if(@ldap_bind($this->conn, $auth_user, $passwd)) {
                @ldap_close($this->conn);
                return TRUE;
            }
        }
        
        @ldap_close($this->conn);
        return FALSE;
    }


    /* ldap server bind */
    function _bind() {
        $this->connType = ($this->config['userName'] && $this->config['userPassword']) ? 1:0; // connection type anonymous or  user|password
        
        ($this->connType) // bind
            ? $this->bind = @ldap_bind($this->conn, $this->config['userName'], $this->config['userPassword'])
            : $this->bind = @ldap_bind($this->conn);

        // bind result
        if ($this->bind) {
            return TRUE ;
        }
        else {
            $this->error = "Cannot bind ldap connection";
            return FALSE;
        }
    }


    /* prepare configuration info */
    function prepare() {

        // prepare import synchronization string 
        $this->pairs = split(',', $this->config['importSynchString']);

        foreach($this->pairs as $pairIndex=> $pairString) {
            $pairDict = split('=', $pairString); // split pair string by '='

            $this->importAttr[$pairIndex] = strtolower($pairDict[1]); // second element as ldap dn attr

            $this->importDict[$pairIndex]['ldapAttr'] = strtolower($pairDict[1]); // first element as sql field
            $this->importDict[$pairIndex]['sqlField'] = $pairDict[0]; // ...
        }
    }

    
    /* import ldap entries */
    function import() {
      
        // get import entries
        $this->searchResult = ldap_search($this->conn, $this->config['distName'], $this->config['filter'], $this->importAttr);
        $this->entries      = ldap_get_entries($this->conn, $this->searchResult);
        $resArray = array();

        // prepare import sql queryies
        $resEntryId = 0;
        for ($entryId = 0; $entryId < $this->entries["count"]; $entryId++) {

            // parse ldap entry and prepare sql fields
            $sqlFields = array();
            $resEntryId++;
            foreach($this->importDict as $elem) {
                
                // if ldap attribute value not null 
                $ldapElemValue = @$this->entries[$entryId][$elem['ldapAttr']][0];
                
                if ($ldapElemValue) {
                    $resArray[$resEntryId][$elem['ldapAttr']] = $ldapElemValue;
                }
            }
            
        }
        
        $this->resDict = $resArray;
      }
    
    /* import ldap entries */
    function export() {
      
        // get import entries
        $this->searchResult = ldap_search($this->conn, $this->config['distName'], $this->config['filter'], $this->importAttr);
        $this->entries      = ldap_get_entries($this->conn, $this->searchResult);
        $resArray = array();

        // prepare import sql queryies
        $resEntryId = 0;
        for ($entryId = 0; $entryId < $this->entries["count"]; $entryId++) {

            // parse ldap entry and prepare sql fields
            $sqlFields = array();
            $resEntryId++;
            foreach($this->importDict as $elem) {
                
                // if ldap attribute value not null 
                $ldapElemValue = @$this->entries[$entryId][$elem['ldapAttr']][0];
                
                if ($ldapElemValue) {
                    $resArray[$resEntryId][$elem['ldapAttr']] = $ldapElemValue;
                }
            }
            
        }
        
        $this->resDict = $resArray;
      }

    // get import sql queries
    function resDict() {
        @ldap_close($this->conn);
        return $this->resDict;

    }

    // get import attributes
    function getImportAttr() {
        return $this->importAttr;
    }


};


     // define ldapConf hash
     $ldapConf = array(
    "protocolVersion"   => $ini["LDAP_PROTOCOL_VERSION"],
    "userName"          => $ini["LDAP_ADMIN_USER_NAME"],
    "userPassword"      => $ini["LDAP_ADMIN_USER_PASSWD"],
    "distName"          => $ini["LDAP_DIST_NAME"],
    "domain"            => $ini["LDAP_DOMAIN"],
    "filter"            => $ini["LDAP_FILTER"],
    "serverHost"        => $ini["LDAP_SERVER_HOST"],
    "serverPort"        => $ini["LDAP_SERVER_PORT"],
    "importSynchString" => $ini["LDAP_IMPORT_SYNCH_STRING"]);

     // declare global class instance $ldap
     global $ldap;
     $ldap = new Ldap($ldapConf);

?>

